;;; -*- Mode: Emacs-Lisp -*-

;;; This is a sample .emacs file.
;;;
;;; The .emacs file, which should reside in your home directory, allows you to
;;; customize the behavior of Emacs.  In general, changes to your .emacs file
;;; will not take effect until the next time you start up Emacs.  You can load
;;; it explicitly with `M-x load-file RET ~/.emacs RET'.
;;;
;;; There is a great deal of documentation on customization in the Emacs
;;; manual.  You can read this manual with the online Info browser: type
;;; `C-h i' or select "Emacs Info" from the "Help" menu.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;			Basic Customization			    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Enable the commands `narrow-to-region' ("C-x n n") and 
;; `eval-expression' ("M-ESC", or "ESC ESC").  Both are useful
;; commands, but they can be confusing for a new user, so they're
;; disabled by default.
(put 'narrow-to-region 'disabled nil)
(put 'eval-expression 'disabled nil)
(setq enable-recursive-minibuffers t)

;;; Define a variable to indicate whether we're running XEmacs/Lucid Emacs.
;;; (You do not have to defvar a global variable before using it --
;;; you can just call `setq' directly like we do for `emacs-major-version'
;;; below.  It's clearer this way, though.)

(defvar running-xemacs (string-match "XEmacs\\|Lucid" emacs-version))

(setq gc-cons-threshold (* 1024 1024))

(if (or (not running-xemacs)
	(and (<= emacs-major-version 19)
	     (<= emacs-minor-version 13)))
    (require 'cl))

;; Make the sequence "C-x w" execute the `what-line' command, 
;; which prints the current line number in the echo area.
(global-set-key "\C-xw" 'what-line)

;; set up the function keys to do common tasks to reduce Emacs pinky
;; and such.

(cond (running-xemacs
       ;; Make F1 invoke help
       (global-set-key 'f1 'help-command)
       ;; Make F2 be `undo'
       (global-set-key 'f2 'undo)
       ;; Make F3 be `find-file'
       (global-set-key 'f3 'find-file)

       ;; Make F4 be "mark", F5 be "copy", F6 be "paste"
       ;; Note that you can set a key sequence either to a command or to another
       ;; key sequence.
       (global-set-key 'f4 'set-mark-command)
       (global-set-key 'f5 "\M-w")
       (global-set-key 'f6 "\C-y")

       ;; Make F7 be `save-file'
       (global-set-key 'f7 'save-file)

       ;; Make F8 be "start macro", F9 be "end macro", F10 be "execute macro"
       (global-set-key 'f8 'start-kbd-macro)
       (global-set-key 'f9 'end-kbd-macro)
       (global-set-key 'f10 'call-last-kbd-macro)

       ;; Here's an alternative binding if you don't use keyboard macros:
       ;; Make F8 be `save-file' followed by `delete-window'.
       ;;(global-set-key 'f8 "\C-x\C-s\C-x0")))
))

(setq has-dired (locate-library "dired"))

(cond (running-xemacs
       ;;
       ;; Code for any version of XEmacs/Lucid Emacs goes here
       ;;

       ;; Change the values of some variables.
       ;; (t means true; nil means false.)
       ;;
       ;; Use the "Describe Variable..." option on the "Help" menu
       ;; to find out what these variables mean.
       (setq find-file-use-truenames nil
	     find-file-compare-truenames t
	     ;; minibuffer-confirm-incomplete t
	     complex-buffers-menu-p t
	     next-line-add-newlines nil
	     mail-yank-prefix "> "
	     kill-whole-line t
	     )

       ;; When running ispell, consider all 1-3 character words as correct.
       (setq ispell-extra-args '("-W" "3"))

       ;; Change the way the buffer name is displayed in the
       ;; modeline.  The variable for this is called
       ;; 'modeline-buffer-identification but was called
       ;; 'mode-line-buffer-identification in older XEmacsen.
;       (if (boundp 'modeline-buffer-identification)
;	   ;; Note that if you want to put more than one form in the
;	   ;; `THEN' clause of an IF-THEN-ELSE construct, you have to
;	   ;; surround the forms with `progn'.  You don't have to
;	   ;; do this for the `ELSE' clause.
;	   (progn
;	     (setq-default modeline-buffer-identification '("XEmacs: %17b"))
;	     (setq modeline-buffer-identification '("XEmacs: %17b")))
;	 (setq-default mode-line-buffer-identification '("XEmacs: %17b"))
;	 (setq mode-line-buffer-identification '("XEmacs: %17b")))

       (cond ((or (not (fboundp 'device-type))
		  (equal (device-type) 'x))
	      ;;
	      ;; Code which applies only when running emacs under X goes here.
	      ;; (We check whether the function `device-type' exists
	      ;; before using it.  In versions before 19.12, there
	      ;; was no such function.  If it doesn't exist, we
	      ;; simply assume we're running under X -- versions before
	      ;; 19.12 only supported X.)

	      ;; Remove the binding of C-x C-c, which normally exits emacs.
	      ;; It's easy to hit this by mistake, and that can be annoying.
	      ;; Under X, you can always quit with the "Exit Emacs" option on
	      ;; the File menu.
	      (global-set-key "\C-x\C-c" nil)

	      ;; Uncomment this to enable "sticky modifier keys" in 19.13
	      ;; and up.  With sticky modifier keys enabled, you can
	      ;; press and release a modifier key before pressing the
	      ;; key to be modified, like how the ESC key works always.
	      ;; If you hold the modifier key down, however, you still
	      ;; get the standard behavior.  I personally think this
	      ;; is the best thing since sliced bread (and a *major*
	      ;; win when it comes to reducing Emacs pinky), but it's
	      ;; disorienting at first so I'm not enabling it here by
	      ;; default.

	      ;;(setq modifier-keys-are-sticky t)

	      ;; This changes the variable which controls the text that goes
	      ;; in the top window title bar.  (However, it is not changed
	      ;; unless it currently has the default value, to avoid
	      ;; interfering with a -wn command line argument I may have
	      ;; started emacs with.)
	      (if (equal frame-title-format "%S: %b")
		  (setq frame-title-format
			;; Change "emacs:" in frame title to hostname
			;; (stripped of full domain)
			(concat
			 (let ((system-name (system-name)))
			   (or (if (string-match "\\." system-name)
				   (substring system-name 0
					      (match-beginning 0)))
			       system-name))
			 ": %b")))
;			 ": %b[%l:%c]")))
;		  (setq frame-title-format
;			(concat "%S: " invocation-directory invocation-name
;				" [" emacs-version "]"
;				(if nil ; (getenv "NCD")
;				    ""
;				  "   %b"))))

	      ;; If we're running on display 0, load some nifty sounds that
	      ;; will replace the default beep.  But if we're running on a
	      ;; display other than 0, which probably means my NCD X terminal,
	      ;; which can't play digitized sounds, do two things: reduce the
	      ;; beep volume a bit, and change the pitch of the sound that is
	      ;; made for "no completions."
	      ;;
	      ;; (Note that sampled sounds only work if XEmacs was compiled
	      ;; with sound support, and we're running on the console of a
	      ;; Sparc, HP, or SGI machine, or on a machine which has a
	      ;; NetAudio server; otherwise, you just get the standard beep.)
	      ;;
	      ;; (Note further that changing the pitch and duration of the
	      ;; standard beep only works with some X servers; many servers
	      ;; completely ignore those parameters.)
	      ;;
;	      (cond (; (string-match ":0" (getenv "DISPLAY"))
;		     (device-sound-enabled-p)
;		     (load-default-sounds))
;		    (t
;		     (setq bell-volume 40)
;		     (setq sound-alist
;			   (append sound-alist '((no-completion :pitch 500))))
;		     ))

	      ;; Make `C-x C-m' and `C-x RET' be different (since I tend
	      ;; to type the latter by accident sometimes.)
	      (define-key global-map [(control x) return] nil)

	      ;; Change the cursor used when the mouse is over a mode line
	      (setq x-mode-pointer-shape "leftbutton")

	      ;; Change the cursor used during garbage collection.
	      ;;
	      ;; Note that this cursor image is rather large as cursors go,
	      ;; and so it won't work on some X servers (such as the MIT
	      ;; R5 Sun server) because servers may have lamentably small
	      ;; upper limits on cursor size.
	      ;; (if (featurep 'xpm)
		  ;; (setq x-gc-pointer-shape
			;; (expand-file-name "trash.xpm" data-directory)))

	      ;; Here's another way to do that: it first tries to load the
	      ;; cursor once and traps the error, just to see if it's
	      ;; possible to load that cursor on this system; if it is,
	      ;; then it sets x-gc-pointer-shape, because we knows that
	      ;; will work.  Otherwise, it doesn't change that variable
	      ;; because we know it will just cause some error messages.
;	      (if (featurep 'xpm)
;		  (let ((file (expand-file-name "recycle.xpm" data-directory)))
;		    (if (condition-case error
;			    (make-cursor file) ;returns a cursor if successful.
;			  (error nil))	    ; returns nil if an error occurred.
;			(setq x-gc-pointer-shape file))))
	 
	      ;; Add `dired' to the File menu
	      (when (and has-dired
			 (fboundp 'add-menu-button))
		(add-menu-item '("File") "Edit Directory" 'dired t))

	      ;; Here's a way to add scrollbar-like buttons to the menubar
	      ;; (add-menu-item nil "Top" 'beginning-of-buffer t)
	      ;; (add-menu-item nil "<<<" 'scroll-down t)
	      ;; (add-menu-item nil " . " 'recenter t)
	      ;; (add-menu-item nil ">>>" 'scroll-up t)
	      ;; (add-menu-item nil "Bot" 'end-of-buffer t)
	      
	      ;; Change the behavior of mouse button 2 (which is normally
	      ;; bound to `mouse-yank'), so that it inserts the selected text
	      ;; at point (where the text cursor is), instead of at the
	      ;; position clicked.
	      ;;
	      ;; Note that you can find out what a particular key sequence or
	      ;; mouse button does by using the "Describe Key..." option on
	      ;; the Help menu.
	      (setq mouse-yank-at-point t)

	      ;; When editing C code, I often like to insert tabs into
	      ;; comments and such.  It gets to be a pain to always
	      ;; have to use `C-q TAB', so I set up a more convenient
	      ;; binding.
;;;	      (define-key '(shift tab) 'self-insert-command)

	      ;; LISPM bindings of Control-Shift-C and Control-Shift-E.
	      ;; Note that "\C-C" means Control-C, not Control-Shift-C.
	      ;; To specify shifted control characters, you must use the
	      ;; more verbose syntax used here.
	      (define-key emacs-lisp-mode-map '(control C) 'compile-defun)
	      (define-key emacs-lisp-mode-map '(control E) 'eval-defun)

	      ))

       ))

;;; Older versions of emacs did not have these variables
;;; (emacs-major-version and emacs-minor-version.)
;;; Let's define them if they're not around, since they make
;;; it much easier to conditionalize on the emacs version.

(if (and (not (boundp 'emacs-major-version))
	 (string-match "^[0-9]+" emacs-version))
    (setq emacs-major-version
	  (string-to-int (substring emacs-version
				    (match-beginning 0) (match-end 0)))))
(if (and (not (boundp 'emacs-minor-version))
	 (string-match "^[0-9]+\\.\\([0-9]+\\)" emacs-version))
    (setq emacs-minor-version
	  (string-to-int (substring emacs-version
				    (match-beginning 1) (match-end 1)))))

;;; Define a function to make it easier to check which version we're
;;; running.

(defun running-emacs-version-or-newer (major minor)
  (or (> emacs-major-version major)
      (and (= emacs-major-version major)
	   (>= emacs-minor-version minor))))

(cond ((and running-xemacs
	    (running-emacs-version-or-newer 19 6))
       ;;
       ;; Code requiring XEmacs/Lucid Emacs version 19.6 or newer goes here
       ;;
       ))

(cond ((>= emacs-major-version 19)
       ;;
       ;; Code for any vintage-19 emacs goes here
       ;;
       ))

(cond ((and (not running-xemacs)
	    (>= emacs-major-version 19))
       ;;
       ;; Code specific to FSF Emacs 19 (not XEmacs/Lucid Emacs) goes here
       ;;
       ))

(cond ((< emacs-major-version 19)
       ;;
       ;; Code specific to emacs 18 goes here
       ;;
       ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;		Customization of Specific Packages		    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Miranova Material File editing mode for Chemron
;(cond (running-xemacs
;      (require 'mat-mode)
;      (if (not (assoc "\\.mat" auto-mode-alist))
;	  (setq auto-mode-alist
;		(append '(("\\.mat" . material-mode))
;			auto-mode-alist)))))

;;; ********************
;;; Load ange-ftp, which uses the FTP protocol as a pseudo-filesystem.
;;; When this is loaded, the pathname syntax /user@host:/remote/path
;;; refers to files accessible through ftp.
;;;
(if (and running-xemacs
	 (not (featurep 'infodock))
	 has-dired
	 (fboundp 'console-on-window-system-p)
	 (console-on-window-system-p))
    (add-hook 'dired-load-hook
              #'(lambda ()
		  (if (locate-library "dired-xemacs-highlight")
		      (progn
			(require 'dired-xemacs-highlight)
			(set-face-foreground 'dired-face-boring
					     "DarkGoldenrod")
			(set-face-foreground 'dired-face-socket
					     "DarkCyan"))
		    (set-face-foreground 'dired-face-permissions "purple")
		    (set-face-foreground 'dired-face-marked "orange red")
		    ;(set-face-foreground 'dired-face-deleted "yellow3")
		    (set-face-foreground 'dired-face-socket "magenta")
		    (set-face-foreground 'dired-face-setuid "red")
		    (set-face-foreground 'dired-face-boring "DarkGoldenrod")
		    (set-face-foreground 'dired-face-symlink "cyan")
		    (set-face-foreground 'dired-face-executable "green")
		    (set-face-foreground 'dired-face-directory "blue")))

              t))

(when (and has-dired (not (featurep 'infodock)))
  (setq dired-do-permission-highlighting-too t)
  (require 'dired))
; (require 'ange-ftp)
; (setq ange-ftp-default-user "anonymous"      ; id to use for /host:/remote/path
;       ange-ftp-generate-anonymous-password t ; use $USER@`hostname`
;       ange-ftp-binary-file-name-regexp "."   ; always transfer in binary mode
;       )


;;; ********************
;;; Load the auto-save.el package, which lets you put all of your autosave
;;; files in one place, instead of scattering them around the file system.
;;;
(cond (running-xemacs
      (require 'auto-save)
      (setq auto-save-directory (expand-file-name "~/autosaves/")
	    auto-save-directory-fallback auto-save-directory
	    auto-save-hash-p nil
;	    ange-ftp-auto-save t
;	    ange-ftp-auto-save-remotely nil
	    ;; now that we have auto-save-timeout, let's crank this up
	    ;; for better interactive response.
	    auto-save-interval 2000
	    )))

;; This adds additional extensions which indicate files normally
;; handled by cc-mode.
;(setq auto-mode-alist
;      (append '(("\\.C\\'"  . c++-mode)
;		("\\.cc\\'" . c++-mode)
;		("\\.hh\\'" . c++-mode)
;		("\\.c\\'"  . c-mode)
;		("\\.h\\'"  . c-mode))
;	      auto-mode-alist))

;; Linux C source
; (autoload 'linux-c-mode "cc-linux")
; (setq auto-mode-alist (cons '("/usr/src/linux.*/.*\\.[ch]\\'" . linux-c-mode)
;			    auto-mode-alist))

;;; ********************
;;; cc-mode (the mode you're in when editing C, C++, and Objective C files)

;; Tell cc-mode not to check for old-style (K&R) function declarations.
;; This speeds up indenting a lot.
(setq c-recognize-knr-p nil)

;; Change the indentation amount to 4 spaces instead of 2.
;; You have to do it in this complicated way because of the
;; strange way the cc-mode initializes the value of `c-basic-offset'.
;;; (add-hook 'c-mode-hook (lambda () (setq c-basic-offset 4)))


;;; ********************
;;; Load a partial-completion mechanism, which makes minibuffer completion
;;; search multiple words instead of just prefixes; for example, the command
;;; `M-x byte-compile-and-load-file RET' can be abbreviated as `M-x b-c-a RET'
;;; because there are no other commands whose first three words begin with
;;; the letters `b', `c', and `a' respectively.
;;;
;;; (cond (running-xemacs
;;;       (load-library "completer")))


;;; ********************
;;; Load crypt, which is a package for automatically decoding and reencoding
;;; files by various methods - for example, you can visit a .Z or .gz file,
;;; edit it, and have it automatically re-compressed when you save it again.
;;; 
(cond (running-xemacs
       (setq crypt-encryption-type 'pgp   ; default encryption mechanism
	     crypt-confirm-password t	   ; make sure new passwords are correct
					;crypt-never-ever-decrypt t  ; if you don't encrypt anything, set this to
					; tell it not to assume that "binary" files
					; are encrypted and require a password.
	     )
;;       (when (fboundp 'auto-compression-mode)
;;	 (auto-compression-mode nil))
       (when (locate-library "crypt")
	 (require 'crypt))))


;;; ********************
;;; Edebug is a source-level debugger for emacs-lisp programs.
;;;
(define-key emacs-lisp-mode-map "\C-xx" 'edebug-defun)


;;; ********************
;;; Font-Lock is a syntax-highlighting package.  When it is enabled and you
;;; are editing a program, different parts of your program will appear in
;;; different fonts or colors.  For example, with the code below, comments
;;; appear in red italics, function names in function definitions appear in
;;; blue bold, etc.  The code below will cause font-lock to automatically be
;;; enabled when you edit C, C++, Emacs-Lisp, and many other kinds of
;;; programs.
;;;
;;; The "Options" menu has some commands for controlling this as well.
;;;

(setq use-steve-font-lock-colors nil)
;; (set-face-background-pixmap 'default
;;			    (expand-file-name "~hniksic/pics/slate.xpm"))
;(when (and (fboundp 'find-face) (find-face 'modeline-mousable))
;  (set-face-background-pixmap 'modeline-mousable [nothing]))
;(when (and (fboundp 'find-face) (find-face 'modeline-buffer-id))
;  (set-face-background-pixmap 'modeline-buffer-id [nothing]))
(when (and (fboundp 'find-face) (find-face 'modeline-mousable))
  (set-face-background-pixmap 'modeline-mousable [nothing]))
(when (and (fboundp 'find-face) (find-face 'modeline-buffer-id))
  (set-face-background-pixmap 'modeline-buffer-id [nothing]))

(cond ((and running-xemacs use-steve-font-lock-colors)

       ;; If you want the default colors, you could do this:
       ;; (setq font-lock-use-default-fonts nil)
       ;; (setq font-lock-use-default-colors t)
       ;; but I want to specify my own colors, so I turn off all
       ;; default values.
       (setq font-lock-use-default-fonts nil)
       (setq font-lock-use-default-colors nil)))
;;;       (require 'html-mode)
(require 'font-lock)
(if (and (not running-xemacs) (fboundp 'font-lock-make-faces))
    (font-lock-make-faces))

;; (cond (running-xemacs
       ;; Mess around with the faces a bit.  Note that you have
       ;; to change the font-lock-use-default-* variables *before*
       ;; loading font-lock, and wait till *after* loading font-lock
       ;; to customize the faces.

       ;; string face is green
(when use-steve-font-lock-colors
(set-face-foreground 'font-lock-string-face "forest green")

       ;; comments are italic and red; doc strings are italic
       ;;
       ;; (I use copy-face instead of make-face-italic/make-face-bold
       ;; because the startup code does intelligent things to the
       ;; 'italic and 'bold faces to ensure that they are different
       ;; from the default face.  For example, if the default face
       ;; is bold, then the 'bold face will be unbold.)
(copy-face 'italic 'font-lock-comment-face)
(cond (running-xemacs
       ;; Underling comments looks terrible on tty's
       (when (featurep 'tty)
	 (set-face-underline-p 'font-lock-comment-face nil 'global 'tty)
	 (set-face-highlight-p 'font-lock-comment-face t 'global 'tty))))

(copy-face 'font-lock-comment-face 'font-lock-doc-string-face)
(set-face-foreground 'font-lock-comment-face "red")

       ;; function names are bold and blue
(copy-face 'bold 'font-lock-function-name-face)
(set-face-foreground 'font-lock-function-name-face "blue")

       ;; misc. faces
(if running-xemacs
    (and (find-face 'font-lock-preprocessor-face) ; 19.13 and above
	 (copy-face 'bold 'font-lock-preprocessor-face))
  (set-face-foreground 'font-lock-reference-face "VioletRed4")
  (global-font-lock-mode t))

(copy-face 'italic 'font-lock-type-face)

(copy-face 'bold 'font-lock-keyword-face)
(set-face-foreground 'font-lock-keyword-face "Purple")
)

(setq font-lock-maximum-decoration t)

;;; ********************
;;; fast-lock is a package which speeds up the highlighting of files
;;; by saving information about a font-locked buffer to a file and
;;; loading that information when the file is loaded again.  This
;;; requires a little extra disk space be used.
;;;
;;; Normally fast-lock puts the cache file (the filename appended with
;;; .flc) in the same directory as the file it caches.  You can
;;; specify an alternate directory to use by setting the variable
;;; fast-lock-cache-directories.

;; Let's use lazy-lock instead.
(when (and running-xemacs (fboundp 'turn-on-fast-lock))
  (add-hook 'font-lock-mode-hook 'turn-on-fast-lock)
  (setq fast-lock-cache-directories '("/home/steve/fastlock"))
  (setq font-lock-region-before -2)
  (setq font-lock-region-after 1)
  (setq font-lock-maximum-size 1024000))

;;; ********************
;;; lazy-lock is a package which speeds up the highlighting of files
;;; by doing it "on-the-fly" -- only the visible portion of the
;;; buffer is fontified.  The results may not always be quite as
;;; accurate as using full font-lock or fast-lock, but it's *much*
;;; faster.  No more annoying pauses when you load files.

;(add-hook 'font-lock-mode-hook 'turn-on-lazy-lock)
;(if (not (boundp 'turn-on-lazy-lock))
;    (autoload 'turn-on-lazy-lock "lazy-lock" nil t))

;; I personally don't like "stealth mode" (where lazy-lock starts
;; fontifying in the background if you're idle for 30 seconds)
;; because it takes too long to wake up again on my piddly Sparc 1+.
;(setq lazy-lock-stealth-time nil)


;;; ********************
;;; func-menu is a package that scans your source file for function
;;; definitions and makes a menubar entry that lets you jump to any
;;; particular function definition by selecting it from the menu.  The
;;; following code turns this on for all of the recognized languages.
;;; Scanning the buffer takes some time, but not much.
;;;
;;; Send bug reports, enhancements etc to:
;;; David Hughes <ukchugd@ukpmr.cs.philips.nl>
;;;
(setq sb-enable-func-menu nil)
(defun sb-enable-func-menu ()
  (require 'func-menu)
  (define-key global-map 'f8 'function-menu)
  ;;(add-hook 'find-file-hooks 'fume-add-menubar-entry)
  (define-key global-map [(control c) x] 'fume-add-menubar-entry)
  (define-key global-map "\C-cg" 'fume-prompt-function-goto)
  (define-key global-map '(shift button3) 'mouse-function-menu)
  ;; For descriptions of the following user-customizable variables,
  ;; type C-h v <variable>
  (setq fume-max-items 25
	fume-fn-window-position 3
	fume-auto-position-popup t
	fume-display-in-modeline-p t
	;; fume-menubar-menu-location "File"
	fume-buffer-name "*Function List*"
	fume-no-prompt-on-valid-default nil)
  )

(when (and running-xemacs
	   sb-enable-func-menu
	   (locate-library "func-menu"))
  (sb-enable-func-menu))

;;; ********************
;;; MH is a mail-reading system from the Rand Corporation that relies on a
;;; number of external filter programs (which do not come with emacs.)
;;; Emacs provides a nice front-end onto MH, called "mh-e".
;;;
;; Bindings that let you send or read mail using MH
;(global-set-key "\C-xm" 'mh-smail)
;(global-set-key "\C-x4m" 'mh-smail-other-window)
;(global-set-key "\C-cr" 'mh-rmail)

;; Customization of MH behavior.
;(setq mh-delete-yanked-msg-window t)
;(setq mh-yank-from-start-of-msg t)
;(setq mh-summary-height 11)

;; Use lines like the following if your version of MH
;; is in a special place.
;; (setq mh-progs "/usr/bin/mh/")
;; (setq mh-lib "/usr/lib/mh/")

;;; Make adaptive Fill be the default.
(when (and (not (featurep 'infodock))
	   (locate-library "filladapt"))
  (require 'filladapt)
  (when (and (boundp 'filladapt-token-table)
	     (rassoc '(supercite-citation) filladapt-token-table))
    (setf (car (rassoc '(supercite-citation) filladapt-token-table))
	  "\\(\\w\\|[0-9]\\)[^'`\"< \t\n]*>[ \t]*"))
  ;(setq-default filladapt-mode t)
  (setq filladapt-mode-line-string " FA")
  ;; Try to smooth things out a bit.
  (setq filladapt-fill-column-tolerance 5))

;;; Set up for printing
;; (setq ps-lpr-switches '("-Plw"))

;; Following two lines are for hecate
;(setq ps-print-color-p t)
;(setq ps-adobe-tag "%!PS-Adobe-3.0\n")

;; (setq lpr-switches '("-Pepson"))
;(setq ps-paper-type 'letter)

;; Vc mode should make backups like every other mode.
(setq vc-make-backup-files t)

(autoload 'irchat "irchat" "GNUemacs IRC client." t)

;;; Ksh-mode enhancements
(autoload 'sh-mode "sh-script" "Shell Script Mode." t)
(add-hook 'ksh-mode-hook
	  #'(lambda ()
	      (setq ksh-indent 8)
	      (setq ksh-group-offset -8)
	      (setq ksh-brace-offset -8)
	      (setq ksh-match-and-tell t)
	      (setq ksh-tab-always-indent t)
	      (font-lock-mode 1)))
(add-hook 'sh-mode-hook
	  #'(lambda ()
	      (setq sh-indentation 8)))
(if (and running-xemacs
	 (<= emacs-major-version 19)
	 (<= emacs-minor-version 14))
    (setq auto-mode-alist (append auto-mode-alist
				  (list
				   '("\\.sh\\'" . ksh-mode)
				   '("\\.ksh\\'" . ksh-mode)
				   '("\\.zsh\\'" . ksh-mode)
				   '("\\.bashrc\\'" . ksh-mode)
				   '("\\.zlogin\\'" . ksh-mode)
				   '("\\.zshrc\\'" . ksh-mode)
				   '("\\.zshenv\\'" . ksh-mode)
				   '("\\.zlogout\\'" . ksh-mode)
				   '("\\..*profile\\'" . ksh-mode))))
  (unless (>= emacs-major-version 20)
    (setq auto-mode-alist (append auto-mode-alist
				  (list
				   '("\\.sh\\'" . sh-mode)
				   '("\\.ksh\\'" . sh-mode)
				   '("\\.zsh\\'" . sh-mode)
				   '("\\.bashrc\\'" . sh-mode)
				   '("\\.zlogin\\'" . sh-mode)
				   '("\\.zshrc\\'" . sh-mode)
				   '("\\.zshenv\\'" . sh-mode)
				   '("\\.zlogout\\'" . sh-mode)
				   '("\\..*profile\\'" . sh-mode))))))


;;; (add-hook 'mh-show-mode-hook 'steves-mh-show-mode-hook)

;;; ********************
;;; resize-minibuffer-mode makes the minibuffer automatically
;;; resize as necessary when it's too big to hold its contents.

(cond ((and running-xemacs
	    (fboundp 'resize-minibuffer-mode))
       ;; (autoload 'resize-minibuffer-mode "rsz-minibuf" nil t)
       (resize-minibuffer-mode)
       (setq resize-minibuffer-window-exactly nil)))

 
;;; ********************
;;; W3 is a browser for the World Wide Web, and takes advantage of the very
;;; latest redisplay features in XEmacs.  You can access it simply by typing 
;;; 'M-x w3'; however, if you're unlucky enough to be on a machine that is 
;;; behind a firewall, you will have to do something like this first:

;(setq w3-use-telnet t
;      ;;
;      ;; If the Telnet program you use to access the outside world is
;      ;; not called "telnet", specify its name like this.
;      w3-telnet-prog "itelnet"
;      ;;
;      ;; If your Telnet program adds lines of junk at the beginning
;      ;; of the session, specify the number of lines here.
;      w3-telnet-header-length 4
;      )

;;  steve customizations here ...
;;; (cond (running-xemacs
       (setq c-font-lock-keywords c-font-lock-keywords-2)
       (setq lisp-font-lock-keywords lisp-font-lock-keywords-2) ;))

(setq load-warn-when-source-newer t)

(make-variable-buffer-local 'compile-command)
(add-hook 'html-mode-hook
	  #'(lambda ()
	      (setq compile-command (concat "weblint "
					    (file-name-nondirectory
					     buffer-file-name)))))

(if (and (featurep 'toolbar)
	 (not (featurep 'infodock))
	 (fboundp 'console-on-window-system-p)
	 (console-on-window-system-p)
	 (file-exists-p (expand-file-name "~/.toolbar")))
    (load-file (expand-file-name "~/.toolbar")))

;; (global-set-key "\C-X\C-E" 'compile)

;(setq mail-self-blind t)

(defun steves-previous-window ()
  "Move backwards to the previous window"
  (interactive)
  (other-window -1))
(global-set-key "\C-xp" 'steves-previous-window)

(defun steves-next-window ()
  "Move forward to the next window"
  (interactive)
  (other-window 1))
(global-set-key "\C-XN" 'steves-next-window)

;(defun steves-c-mode-hook ()
;  "Steve's C-Mode initialization"
;  (if (or (not (boundp 'compile-command)) (null compile-command))
;      (setq compile-command (concat
;			     "make -ke DEBUGFLAGS=\"-g -DIO_DEBUG\" "
;			     (substring
;			      (file-name-nondirectory buffer-file-name) 0 -2)
;			     ".o")))
;  (setq c-indent-level 8)
;  (setq c-basic-offset 8)
;  (setq c-continued-statement-offset 8)
;  (setq c-brace-offset -8)
;  (setq c-argdecl-indent 8)
;  (setq c-comment-indent 8)
;  (setq c-label-offset -8))

; (add-hook 'c-mode-hook 'steves-c-mode-hook)

(add-hook 'c-mode-hook
	  (lambda ()
	    (if (string-match "\\`\\(/usr/src\\|/home/xemacs\\|/b/XEmacs\\)/xemacs.*\\'" buffer-file-name)
		(c-set-style "gnu")
	      (c-set-style "linux"))))

(defun steves-perl-mode-hook ()
  "Customize indentation for perl code"
  (setq perl-indent-level 8)
  (setq perl-continued-statement-offset 8)
  (setq perl-continued-brace-offset 0)
  (setq perl-brace-offset -8)
  (setq perl-brace-imaginary-offset 0)
  (setq perl-label-offset -8))

(add-hook 'perl-mode-hook 'steves-perl-mode-hook)

(fset 'backward-delete-char-untabify 'backward-delete-char)
;;; I despise this function ...
;(defun backward-delete-char-untabify (arg &optional killp)
;  "Delete characters backward, preserving tabs.
;Delete ARG chars, and kill (save in kill ring) if KILLP is non-nil.
;Interactively, ARG is the prefix arg (default 1)
;and KILLP is t if a prefix arg was specified."
;  (interactive "*p\nP")
;  (let ((count arg))
;    (save-excursion
;      (while (and (> count 0) (not (bobp)))
;	;; (if (= (preceding-char) ?\t)
;	    ;; (let ((col (current-column)))
;	      ;; (forward-char -1)
;	      ;; (setq col (- col (current-column)))
;	      ;; (insert-char ?\ col)
;	      ;; (delete-char 1)))
;	(forward-char -1)
;	(setq count (1- count)))))
;  (delete-backward-char arg killp)
;  ;; In overwrite mode, back over columns while clearing them out,
;  ;; unless at end of line.
;  (and overwrite-mode (not (eolp))
;       (save-excursion (insert-char ?\  arg))))


;;(setq lpr-switches '("-Plp"))

;(setq-default desktop-locals-to-save
;      (list 'desktop-locals-to-save		; Itself!  Think it over.
;	    'truncate-lines
;	    'case-fold-search
;	    'case-replace
;	    'fill-column
;	    'overwrite-mode
;	    'change-log-default-name
;	    'line-number-mode
;	    'compile-command))

;(load "desktop")
;(add-hook 'kill-emacs-hook
;	  '(lambda ()
;	    (desktop-truncate search-ring 3)
;	    (desktop-truncate regexp-search-ring 3)))

(defun directory-in-load-path-p (dir)
  (delq nil
	(mapcar (lambda (directory)
		  (when (string-match "/\\([^/]+\\)/?\\'" directory)
		    (string= dir
			     (substring directory
					(match-beginning 1)
					(match-end 1)))))
		load-path)))

;;; Gnus stuffs
(setq slb-gnus-lockfile "~/.gnus-lockfile")
(setq gnus-startup-file-coding-system 'iso-2022-7bit)
(when (and (or (not running-xemacs)
	       (>= emacs-major-version 20)
	       (> emacs-minor-version 14))
	   (or running-xemacs
	       (< emacs-major-version 20))
	   (not (file-exists-p slb-gnus-lockfile)))
  (setq custom-file "~/.gnus-custom")
  (when (and (not (featurep 'infodock))
	     (fboundp 'custom-set-faces)
	     (file-exists-p custom-file))
    (load custom-file))

  (add-hook 'gnus-startup-hook
	    (lambda ()
	      (condition-case nil
		  (make-directory slb-gnus-lockfile)
		(t nil))))
  (add-hook 'gnus-after-exiting-gnus-hook
	    (lambda ()
	      (condition-case nil
		  (delete-directory slb-gnus-lockfile)
		(t nil))))
  (setq gnus-use-installed-gnus t)
  (setq gnus-use-installed-mailcrypt t)
  ;; (when (setq gnus-use-bbdb (locate-library "bbdb"))
  ;;   (setq gnus-bbdb-lisp-directory (file-name-directory gnus-use-bbdb)))
  (setq gnus-use-bbdb nil)
  (when (fboundp 'bbdb-initialize)
    (bbdb-initialize 'gnus 'sc))
  (setq gnus-use-sc (fboundp 'sc-cite-original))

  ;; For testing a distribution
  ;(setq gnus-use-installed-gnus t)
  ;(setq gnus-use-installed-mailcrypt t)
  ;(setq gnus-use-bbdb nil)
  ;(setq custom-file "~/.custom.el")
  ;(when (featurep 'infodock)
    ;(setq gnus-experimental t))

  (setq gnus-use-tm nil)

  (unless (or (featurep 'infodock)
	      (not (directory-in-load-path-p "pgnus")))
    (setq gnus-experimental t)
    (setq gnus-use-tm nil))

  (when (or (featurep 'infodock)
	    (and (= emacs-major-version 19)
		 (>= emacs-minor-version 15))
	    (and (= emacs-major-version 20)
		 (>= emacs-minor-version 3)))
    (setq gnus-use-installed-gnus t))

  (setq gnus-init-file "~/.gnus5.2")
  ;; (setq gnus-play-startup-jingle t)
  (if running-xemacs
      (progn
;	(setq gnus-bbdb-lisp-directory "/usr/lib/xemacs/bbdb-1.51/")
	(when (and (>= emacs-major-version 20)
		   (featurep 'mule))
	  (setq code-converter-is-broken nil)
	  )
	(setq steve-gnus-path "/usr/lib/xemacs/gnus-5.4.53/lisp/")
	)
    (setq steve-gnus-path "/usr/local/share/emacs/site-lisp/rgnus/lisp/")
;    (setq gnus-bbdb-lisp-directory
;	  "/usr/local/share/emacs/site-lisp/bbdb-1.51/")
    (setq gnus-use-mailcrypt t)
    (setq gnus-use-tm t)
    (setq gnus-tm-lisp-directory "/usr/local/share/emacs/site-lisp/tm/")
    )

  (setq gnus-gnus-lisp-directory steve-gnus-path)

  ;; (setq gnus-use-mailcrypt nil)

  (setq gnus-use-mailcrypt nil)
  (setq mime-setup-use-signature nil)
  (setq sc-confirm-always-p nil)

  (load "gnus-setup")
  ;; Rmail stuffs
  (setq mail-archive-file-name "~/spool/sent.spool")

  (if gnus-use-tm
      (progn
	(setq mime-editor/message-default-max-length 10000)
	(setq mime-editor/message-max-length-alist
	      '((news-reply-mode . 10000)))
	(require 'tl-misc)
	(call-after-loaded 'tm-view
			   #'(lambda () (require 'tm-pgp))))))

;;; Avoid an XEmacs bug, do not invert colors when fetching a password
(setq passwd-invert-frame-when-keyboard-grabbed nil)

(setq pixel-vertical-clip-threshold 0)

;; Set up colors for FSFmacs
(unless running-xemacs
  (set-face-foreground 'modeline "black")
  (set-face-background 'modeline "grey85")
  (set-face-foreground 'region "black")
  (set-face-background 'region "grey95")
  (transient-mark-mode t)
  (setq highlight-wrong-size-font nil)
  (global-set-key [C-left] #'(lambda () (interactive) (scroll-right 10)))
  (global-set-key [C-right] #'(lambda () (interactive) (scroll-left 10))))

(when (and (fboundp 'console-on-window-system-p)
	   (console-on-window-system-p)
	   (or (fboundp 'custom-set-face-font-family)
	       (featurep 'cus-face)
	       (and (locate-library "cus-face")
		    (require 'cus-face))))
  ;; (message "Helvetica")
  (custom-set-face-font-family 'modeline "Helvetica"))

;;; Tcl Mode stuffs
(setq tcl-default-application "wishx")
(setq tcl-application "wishx")
(setq next-line-add-newlines nil)
(setq scroll-step 1)

;;; HTML/SGML stuffs
; PSGML settings from m.koster@nexor.co.uk                               
; For more info see <http://www.lysator.liu.se/projects/about_psgml.html>
(if (and running-xemacs
	 (<= emacs-major-version 19)
	 (<= emacs-minor-version 13))
    (progn
      (autoload 'sgml-mode "psgml" "Major mode to edit SGML files." t )

      (setq sgml-always-quote-attributes t)      ; expected by many clients
      (setq sgml-auto-insert-required-elements t)
      (setq sgml-indent-data t)
      (setq sgml-indent-step 2)
      (setq sgml-suppress-warning t)
      (setq sgml-auto-activate-dtd t)            ; preload dtd
      (setq sgml-omittag nil)
      (setq sgml-shorttag nil)
      (setq sgml-recompile-out-of-date-cdtd nil)
   
					; menus for creating new documents
      (setq sgml-custom-dtd '                                     
	    (
	     ( "HTML 2.0"
               "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">" )
	     ( "HTML 2.0 Level 1"
               "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0 Level 1//EN\">" )

	     ( "HTML 2.0 Strict"
               "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0 Strict//EN\">" )

	     ( "HTML 2.0 Strict Level 1"
               "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0 Strict Level 1//EN\">")
	     ( "HTML 3.0"
               "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 3.0//EN//\" [ <!ENTIT Y % HTML.Recommended \"INCLUDE\"> ] >")))

      (defun html-mode ()
	(interactive)
	(sgml-mode)
	(make-local-variable 'sgml-declaration)
	(make-local-variable 'sgml-default-doctype-name)
	(setq sgml-declaration             "/usr/local/lib/sgml/html.decl"
	      sgml-default-doctype-name    "html"
	      sgml-always-quote-attributes t
	      sgml-indent-step             2
	      sgml-indent-data             t
	      sgml-minimize-attributes     nil
	      sgml-omittag                 t
	      sgml-recompile-out-of-date-cdtd nil
	      sgml-shortag                 t  )))
  (setq html-helper-do-write-file-hooks nil)
  (setq sgml-fold-menu
	'("Fold"
	  ["Fold Element" sgml-fold-element t]
	  ["Fold Subelement" sgml-fold-subelement t]
	  ["Fold Region" sgml-fold-region t]
	  ["Unfold Line" sgml-unfold-line t]
	  ["Unfold Element" sgml-unfold-element t]
	  ["Unfold All" sgml-unfold-all t]
	  ["Expand" sgml-expand-element t]
	  ["----------" nil nil]
	  ["Hide Attributes" sgml-hide-attributes t]
	  ["Hide Tags" sgml-hide-tags t]
	  ["Show Attributes" sgml-show-attributes t]
	  ["Show Tags" sgml-show-tags t]
	  )))

;(setq load-path (cons "/usr/lib/xemacs/psgml-xemacs-19.15" load-path))
;(when (and running-xemacs)
;  (setq sgml-ecat-files (list "/home/steve/sgml/ECAT"))
;  (setq sgml-catalog-files (list "/home/steve/sgml/CATALOG")))

(setq html-helper-htmldtd-version
      "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\"\n\t\t\"http://www.w3.org/TR/REC-html40/strict.dtd\">\n")

(setq html-helper-new-buffer-template
      '(html-helper-htmldtd-version
	"<HTML>\n"
	"<HEAD>\n"
	"<TITLE>" (p "Document Title: " title) "</TITLE>\n"
	"</HEAD>\n\n"
	"<BODY>\n"
	"<H1>" (s title) "</H1>\n\n"
	p
	"\n</BODY>\n</HTML>\n"))
(setq-default html-helper-new-buffer-template
      '(html-helper-htmldtd-version
	"<HTML>\n"
	"<HEAD>\n"
	"<TITLE>" (p "Document Title: " title) "</TITLE>\n"
	"</HEAD>\n\n"
	"<BODY>\n"
	"<H1>" (s title) "</H1>\n\n"
	p
	"\n</BODY>\n</HTML>\n"))

;(setq sgml-validate-command
;      "sgmls-1.1.91 -s -m /home/steve/sgml/CATALOG %s %s")
;(setq sgml-data-directory "/home/steve/sgml")

; End of PSGML settings
; (Well, you need to sort out the auto-mode-alist

(unless (>= emacs-major-version 20)
  (setq auto-mode-alist
	(append '(
		  ("\\.sgml\\'" . sgml-mode)
		  ("\\.html\\'" . html-mode)
		  ("\\.htm\\'" .  html-mode)
		  ("\\.html3\\'" . sgml-mode)
		  ("\\.ht3\\'" .   sgml-mode)
		  )
		auto-mode-alist
		)
	))


;;; W3 stuffs
;(setq w3-default-homepage "http://www.miranova.com/links.html")
(setq url-privacy-level 'paranoid
      w3-do-scripting nil)

;;; Debugger stuffs
(define-key emacs-lisp-mode-map "\C-xx" 'edebug-eval-top-level-form)
(autoload 'edebug-eval-top-level-form "edebug")

;;; Miscellaneous stuffs
(when (fboundp 'icomplete-mode)
  (icomplete-mode)
  (setq teach-extended-commands-p nil))

;; ChangeLogs
(setq add-log-full-name "SL Baur")
(setq add-log-mailing-address "steve@xemacs.org")

(add-hook 'change-log-mode-hook 'turn-on-auto-fill)

;(add-hook 'change-log-mode-hook
;	  (lambda ()
;	    (make-local-variable 'add-log-mailing-address)
;	    (cond ((null (stringp (buffer-file-name)))
;		   (setq add-log-mailing-address (concat "steve@"
;							 (system-name))))
;		  ((string-match "/infodock/" (buffer-file-name))
;		   (setq add-log-mailing-address "steve@beopen.com"))
;		  (t
;		   (setq add-log-mailing-address (concat "steve@"
;							 (system-name)))))))
		   
(when (boundp 'toolbar-print-command)
  (setq toolbar-print-command 'ps-print-buffer-with-faces))

(unless (or (and (featurep 'infodock)
		 (boundp 'infodock-major-version)
		 (>= infodock-major-version 3)
		 (>= infodock-minor-version 90)
		 (< infodock-build-version 4))
	    (not (fboundp 'defadvice)))
(defadvice save-buffers-kill-emacs (around save-buffers-kill-emacs freeze)
  "Killing XEmacs is usually an unwise thing to do.
Make it just a little bit harder to do by accident."
  (interactive "P")
  (let ((msg (concat "Really exit "
		     (if (featurep 'infodock)
			 "InfoDock"
		       "XEmacs")
		     "? ")))
    (when (cond ((fboundp 'yes-or-no-p-minibuf)
		 (yes-or-no-p-minibuf msg))
		(t
		 (yes-or-no-p "Really exit GNU Emacs? ")))
      ad-do-it)))
)

(unless (featurep 'scroll-in-place)
  (let ((sip (locate-library "scroll-in-place")))
    (when sip
      (setq scroll-allow-blank-lines-past-eob t)
      (load sip nil t))))

(if running-xemacs
    (if (boundp 'signal-error-on-buffer-boundary)
	(setq signal-error-on-buffer-boundary nil)
      ;; Make scroll-up and scroll-down work sensibly
      (defadvice scroll-up (around scroll-up freeze)
	(interactive "_P")
	(let ((zmacs-region-stays t))
	  (if (interactive-p)
	      (condition-case nil
		  ad-do-it
		(end-of-buffer (goto-char (point-max))))
	    ad-do-it)))

      (defadvice scroll-down (around scroll-down freeze)
	(interactive "_P")
	(let ((zmacs-region-stays t))
	  (if (interactive-p)
	      (condition-case nil
		  ad-do-it
		(beginning-of-buffer (goto-char (point-min))))
	    ad-do-it))))
  (defadvice scroll-up (around scroll-up freeze)
    (interactive "P")
    (let ((zmacs-region-stays t))
      (if (interactive-p)
	  (condition-case nil
	      ad-do-it
	    (end-of-buffer (goto-char (point-max))))
	ad-do-it)))

  (defadvice scroll-down (around scroll-down freeze)
    (interactive "P")
    (let ((zmacs-region-stays t))
      (if (interactive-p)
	  (condition-case nil
	      ad-do-it
	    (beginning-of-buffer (goto-char (point-min))))
	ad-do-it))))

;(when (and running-xemacs
;	   ; (<= emacs-major-version 19)
;	   (not (member "/usr/local/lib/xemacs/site-lisp/" load-path)))
;  (setq load-path (cons "/usr/local/lib/xemacs/site-lisp/" load-path)))

(when (and window-system
	   (locate-library "mic-paren"))
  (setq paren-face 'bold)
  (setq paren-sexp-mode t)
  (setq paren-priority 'open)
  (setq paren-delay nil)
  (setq paren-dont-touch-blink t)
  (setq paren-dont-activate-on-load nil)
  (require 'mic-paren))

(add-hook 'enriched-mode-hook
	  #'(lambda ()
	      (and (fboundp 'turn-off-font-lock)
		   (turn-off-font-lock))))

(when (boundp 'focus-follows-mouse)
  (setq focus-follows-mouse t))

(unless (fboundp 'turn-on-eldoc-mode)
  (when (locate-library "eldoc")
    (autoload 'turn-on-eldoc-mode "eldoc" nil t)))

(when (fboundp 'turn-on-eldoc-mode)
  (add-hook 'emacs-lisp-mode-hook 'turn-on-eldoc-mode)
  (add-hook 'lisp-interaction-mode-hook 'turn-on-eldoc-mode))

(when (fboundp 'turn-on-font-lock)
  (add-hook 'lisp-interaction-mode-hook 'turn-on-font-lock))

(defun sb-init-apropos-faces ()
  (setq apropos-symbol-face (make-face 'apropos-symbol-face))
  (set-face-foreground 'apropos-symbol-face "Purple")
  (when (featurep 'xemacs)
    (make-face-bold 'apropos-symbol-face))
  (setq apropos-keybinding-face (make-face 'apropos-keybinding-face))
  (set-face-foreground 'apropos-keybinding-face "Blue")
  (set-face-underline-p 'apropos-keybinding-face t)
  (setq apropos-label-face (make-face 'apropos-label-face))
  (set-face-foreground 'apropos-label-face "DeepPink")
  (and running-xemacs (make-face-italic 'apropos-label-face))
  ;; Yuck! #### FIXME in apropos.el
  (setq apropos-label-face `(face ,apropos-label-face
				  mouse-face highlight))
  (setq apropos-property-face (make-face 'apropos-property-face))
  (set-face-foreground 'apropos-property-face "Yellow4")
  (and running-xemacs (make-face-bold-italic 'apropos-property-face))
  (when (boundp 'apropos-mode-hook)
    (remove-hook 'apropos-mode-hook 'sb-init-apropos-faces)))

(when (or running-xemacs
	  (not (featurep 'infodock))
	  window-system)
  (if running-xemacs
      (add-hook 'apropos-mode-hook 'sb-init-apropos-faces)
    (sb-init-apropos-faces)))

(setq help-selects-help-window nil)
(setq pre-display-buffer-function nil)

;; Not needed any more
;(defun enable-wnn ()
;  "Enable Wnn/Egg input method."
;  (interactive)
;  (when (and (featurep 'wnn)
;	     (not (featurep 'steve-wnn)))
;    (provide 'steve-wnn)
;    (setq wnn-english-messages t)
;    (require 'egg)
;    (load-library "egg-wnn")
;    (setq egg-default-startup-file "eggrc-wnn")
;    (setq wnn-server-type 'jserver)
;    (set-jserver-host-name "localhost")
;    (unless (featurep 'egg-jpn)
;      (provide 'egg-jpn)
;      (load "its/its-hira.el" nil t)
;      (load "its/its-kata.el" nil t)
;      (load "its/its-hankaku" nil t)
;      (load "its/its-zenkaku" nil t)
;      (setq its:*standard-modes*
;	    (append
;	     (list (its:get-mode-map "roma-kana")
;		   (its:get-mode-map "roma-kata")
;		   (its:get-mode-map "downcase")
;		   (its:get-mode-map "upcase")
;		   (its:get-mode-map "zenkaku-downcase")
;		   (its:get-mode-map "zenkaku-upcase"))
;	     its:*standard-modes*))
;      (setq-default its:*current-map* (its:get-mode-map "roma-kana")))))

;; (setq canna-server "hecate.xemacs.org")

;; Not needed any more
;(defun enable-canna ()
;  "Enable Canna input method."
;  (interactive)
;  (when (boundp 'CANNA)
;    (require 'canna)
;    ;(global-set-key [(control ?\\)] 'canna-toggle-japanese-mode)
;    (canna)
;    ;(global-set-key [(control ?o)] 'open-line)
;    ))

(setq display-time-display-pad 'transparent)
(setq display-time-form-list '(date time load))

(when (and (featurep 'auctex-autoloads)
	   (not (featurep 'infodock)))
  (require 'tex-site))

(if (fboundp 'transpose-preceding-chars)
    (fset 'gosmacs-transpose-chars 'transpose-preceding-chars)
  (defun gosmacs-transpose-chars (arg)
    "Interchange characters before point.
With prefix arg ARG, effect is to take character before point
and drag it forward past ARG other characters (backward if ARG negative).
If no argument and not at start of line, the previous two chars are exchanged."
    (interactive "*P")
    (and (null arg) (not (bolp)) (forward-char -1))
    (transpose-subr 'forward-char (prefix-numeric-value arg))))

(define-key global-map [(control t)] 'gosmacs-transpose-chars)

(when (and running-xemacs
	   (fboundp 'console-on-window-system-p)
	   (console-on-window-system-p)
	   (fboundp 'display-time)
	   (not (featurep 'infodock)))
  (display-time))

(when (fboundp 'find-library)
  (define-key global-map "\C-xl" 'find-library)
  (define-key global-map "\C-xL" 'count-lines-page))

;(when (fboundp 'set-language-environment)
;  (set-language-environment 'european))

(when (and (featurep 'mule) (fboundp 'locate-data-directory))
  (setq skk-tut-file (concat (locate-data-directory "skk") "SKK.tut.E")))

;(when (and (featurep 'mule)
;	   (fboundp 'console-on-window-system-p)
;	   (console-on-window-system-p))
;  (set-face-font
;   'default
;   "-mnkaname-*-*-*-*--12-*-*-*-*-*-jisx0208.1983-0"
;   'global
;   '(mule-fonts) 'prepend))

(when (and (featurep 'mule)
	   (fboundp 'console-on-window-system-p)
	   (console-on-window-system-p))
  (set-face-font
   'default
   ;; "-misc-*-medium-*-*--14-*-*-*-*-*-jisx0208.1983-0"
   ;; "-misc-*-medium-*-*--14-*-*-*-*-*-*-0"
   '(;; "-misc-fixed-medium-r-normal--0-0-75-75-c-0-jisx0208.1983-0"
     "-jis-gothic-medium-r-normal--18-170-75-75-c-180-jisx0208.1983-0"
     "-kappa-fixed-medium-r-normal--20-190-75-75-c-200-jisx0208.1983-0"
     "-JIS-Fixed-Medium-R-Normal--16-150-75-75-C-160-JISX0208.1983-0"
     "-jis-fixed-medium-r-normal--16-110-100-100-c-160-jisx0208.1983-0")
   'global
   '(mule-fonts) 'prepend))


(when (and (featurep 'mule)
	   (fboundp 'lookup))
  (setq lookup-search-agents
	'((ndeb "/usr/local/dict")))
  (define-key global-map [(control ?c) ?& ?l] 'lookup-region))

(add-hook 'Info-mode-hook
	  (lambda () (remove-hook 'mouse-track-click-hook
				  'Info-maybe-follow-clicked-node)))

(add-hook 'outline-mode-hook
	  (lambda ()
	    (when (fboundp 'filladapt-mode)
	      (filladapt-mode))
	    (auto-fill-mode t)
	    (when (fboundp 'turn-on-font-lock)
	      (turn-on-font-lock))))

(defvar steve-hotlist
  '(("J Orlin Grabbe" . "http://www.aci.net/kalliste/")
    ("Zola Times" . "http://www.zolatimes.com/")
    ("Pgcc" . "http://www.goof.com/pcg/")
    ("Linux Maintenance Project" . "http://linux.dialnet.net/LMP")
    ("Lesstif" . "ftp://ftp.hungry.com/pub/hungry/lesstif/")
    ("Currency converter" . "http://www.xe.net/currency/")
    ("Screen It" . "http://www.screenit.com/")
    ("Michael Reagan" . "http://www.reagan.com/")
    ("Ken Hamblin" . "http://hamblin.com/")
    ("Art Bell" . "http://www.artbell.com/")
    ("Weather Underground" . "http://www.princeton.edu/Webweather/ww.html")
    ("Libertarian Party" . "http://www.access.digex.net/~lphq/lphq.html")
    ("CORBA" . "http://linas.org/linux/corba.html")
    ("Washington Post" . "http://www.washingtonpost.com/")
    ("CNN" . "http://www.cnn.com/")
    ("All Politics" . "http://www.allpolitics.com/")
    ("ABC News" . "http://www.abcnews.com/")
    ("MS NBC" . "http://www.msnbc.com/")
    ("Cyphernomicon" . "http://www.oberlin.edu/~brchkind/cyphernomicon/")
    ("Icon" . "http://www.cs.arizona.edu/icon/www/")
    ("Nethack" . "http://pinky.wtower.com/nethack/")
    ("Vortex/Boomerang drivers" . "http://cesdis.gsfc.nasa.gov/linux/drivers/vortex.html")
    ("Linux Big Mama" . "http://www.linuxmama.com/")
    ("Glibc" . "http://www.imaxx.net/~thrytis/glibc/")
    ("Free CORBA" . "http://adams.patriot.net/~tvalesky/freecorba.html")
    ("TAZ" . "http://www.notam.uio.no/~mariusw/bey/taz/taz_contents.html")
    ("GIMP" . "http://xach.dorknet.com/gimp/news/")
    ("Earthquakes" . "http://www.syzygyjob.com/")
    ("New World Order" . "http://www.inforamp.net/~jwhitley")
    ("Sightings" . "http://www.sightings.com/")
    ("Drudge Report" . "http://www.drudgereport.com/")
    ("Linux PNP Patch" . "http://bjesomar.srce.hr/~zcalusic/linux-pnp/")
    ("Reverse Speech" . "http://www.reversespeech.com/")
    ("Linux Weekly News" "http://www.lwn.net/")
))

(when (fboundp 'webjump)
  (require 'webjump)
  (setq webjump-sites
	(append webjump-sample-sites
		steve-hotlist)))

(when (featurep 'mule)
  ;; edict
  (unless (assoc "edict$" file-coding-system-alist)
    (setq file-coding-system-alist
	  (append file-coding-system-alist '(("edict$" . euc-japan)))))
  (setq edict-use-electric-henkan 'yes)
  (setq edict-dictionaries nil)
  (when (file-exists-p "/usr/local/lib/edict")
    (push "/usr/local/lib/edict" edict-dictionaries))
  (when (file-exists-p "/usr/share/misc/xjdic/edict_composite")
    (push "/usr/share/misc/xjdic/edict_composite" edict-dictionaries))

  ;; set up some handy keys
  (when edict-dictionaries
    (define-key global-map [(control ?c) ?& ?e] 'edict-search-english)
    (define-key global-map [(control ?c) ?& ?k] 'edict-search-kanji)))

(random t)

(when (fboundp 'tetris)
  (setq tetris-score-file (expand-file-name "tetris-scores"
					    "~/XEmacs/xemacs-admin")))

(when (fboundp 'xmine)
  (setq xmine-play-sounds (device-sound-enabled-p))
  (setq xmine-width 28)
  (setq xmine-height 35))

(setq user-mail-address "steve@etl.go.jp")
(setq query-user-mail-address nil)

;; EFnet
;; (setq zenirc-server-default "irc.texas.net")
;; Undernet
(setq zenirc-server-default "elric.internetconnect.net")
(setq zenirc-nick-default "altair")

(setq irchat-server "elric.internetconnect.net")
(setq irchat-nickname "altair")

(when (featurep 'infodock)
  (setq id-compression 'none)
  (setq id-pc-regions nil)
  (setq id-resizable-minibuffers nil)
  (setq id-save-configuration nil)
  (setq id-font-lock-style 'none)
  (setq id-customize-gnus nil)
  (setq id-customize-html nil)
  (setq font-lock-auto-fontify t))

(delete* '(-3 . "%p") default-mode-line-format :test 'equal)

;; fuck everyone who likes this worthless shit
(setq delete-key-deletes-forward nil)

(defun sb-snarf-ssh-agent-file ()
  (interactive)
  (let* ((hostname (replace-in-string (system-name) "\\.\\(.*\\'\\)" ""))
	 (ssh-agent-file (expand-file-name (concat "ssh-agent." hostname)
                                           "~/")))
    (when (file-exists-p ssh-agent-file)
      (message "Using %s ..." ssh-agent-file)
      (with-temp-buffer
	(insert-file-contents-literally ssh-agent-file)
	(goto-char (point-min))
	(re-search-forward "\\(.*\\)=\\(/.*\\); .*")
	(replace-match "(setenv \\\"\\1\\\" \\\"\\2\\\")")
	(forward-line 1)
	(beginning-of-line)
	(re-search-forward "\\(.*\\)=\\(.*[0-9]\\);.*")
	(replace-match "(setenv \\\"\\1\\\" \\\"\\2\\\")")
	(forward-line 1)
	(beginning-of-line)
	(re-search-forward "echo \\(.*\\);")
	(replace-match "(message \\\"\\1\\\")")
        ;; (write-file "/tmp/fred")
	(eval-current-buffer)))))

;; Options Menu Settings
;; =====================
(cond
 ((and (string-match "XEmacs" emacs-version)
       (boundp 'emacs-major-version)
       (or (and (= emacs-major-version 19)
                (>= emacs-minor-version 12))
           (>= emacs-major-version 20))
       (featurep 'x)
       use-steve-font-lock-colors)
  (make-face 'zmacs-region)
  (add-spec-list-to-specifier (face-property 'zmacs-region 'background) '((global (nil . "gray90") ((grayscale) . "gray3") ((color) . "gray"))))
  (add-spec-list-to-specifier (face-property 'zmacs-region 'background-pixmap) '((global ((mono x) . [xbm :file "/usr/include/X11/bitmaps/gray3" :data (4 4 "")]) ((mono tty) . [string :data "gray3"]) ((grayscale x) . [nothing]) ((grayscale tty) . [nothing]) ((grayscale stream) . [nothing]) ((color x) . [nothing]) ((color tty) . [nothing]) ((color stream) . [nothing]))))
  (add-spec-list-to-specifier (face-property 'zmacs-region 'reverse) '((global ((tty) . t))))
  ))
;; ============================
;; End of Options Menu Settings
;; (when (not (and (fboundp 'console-on-window-system-p) (console-on-window-system-p)))
;;  (add-hook 'term-setup-hook
;;	'(lambda () (progn
;;		     (set-terminal-coding-system 'euc-jp)
;;		     (set-keyboard-coding-system 'euc-jp))))
;;)

(defun mangle-csv (file)
  "Process an MS DOS CSV file stripping quotes and carriage control."
  (interactive "fCSV File: ")
  (find-file file 'binary)

  ;; step 1 -- strip DOS carriage control
  (goto-char (point-min))
  (while (search-forward "" nil t)
    (replace-match ""))

  ;; step 2 -- strip surrounding double quotes
  (goto-char (point-min))
  (forward-line 1)
  (while (re-search-forward "^\"\\(.*\\)\n\"" nil t)
    (replace-match (match-string 1)))

  ;; step 3 -- adjust quoted quotes in XEmacs names
  (goto-char (point-min))
  (while (search-forward "\"\"" nil t)
    (replace-match "\""))

  ;; step 4 -- delete any extraneous blank lines still left
  (goto-char (point-min))
  (while (re-search-forward "^$" nil t)
    (delete-char -1 nil))

  ;; step 5 -- kill "Body" if it exists
  (goto-char (point-min))
  (when (looking-at "\"Body\"\n")
    (replace-match ""))

  (save-buffer))

;(load "/users/stevbaur/lisp.6/dotemacs-sample")
(add-hook 'eci-load-hooks
	  #'(lambda ()
	      (fset 'eci-internal-log-usage
		    #'(lambda ()
			(message "no logging for this session")))))
(setq eci-custom-queries
      '(("Siebel cli"
	 "[Product]='siebel' and ([Component]='ui' or [Component]='cli')")
	("Siebel ui"
	 "[Product]='siebel' and [Component] like 'ui*'")
	("Production CLI Bugs"
	 "[Identifier]='CSCsi72580' OR \
[Identifier]='CSCsi72922' OR \
[Identifier]='CSCsi51201' OR \
[Identifier]='CSCsi51175' OR \
[Identifier]='CSCsi44438' OR \
[Identifier]='CSCsi02854'")
	("Bugs I'm interested in"
	 "[Identifier]='CSCqd58779' OR \
[Identifier]='CSCqe10310'")
	("Open Bugs (Cat4000)"
	 "[Component]='cat4000' AND [Severity]<= '5' AND \
([Status] = 'N' OR [Status] = 'O' OR [Status] = 'A' OR \
[Status] = 'I' OR [Status] = 'W')")
))
;(setq eci-internal-debugging t) ; no logging
(setq eci-only-1-error nil) ; keep all error messages
(setq browse-url-generic-program "firefox")
(setq browse-url-browser-function 'browse-url-generic)
